﻿#pragma TextEncoding="UTF-8"
#pragma rtGlobals=3
#pragma ModuleName=ScaleSub
#pragma IgorVersion=7
#pragma version=1.06

#include <WaveSelectorWidget>
#include <Resize Controls>

static constant kDefaultOverwrite=1

// https://www.wavemetrics.com/user/tony
// GUI for subtraction of a reference wave scaled by some user-defined factor

menu "Analysis"
	"Wave Scale and Subtract", /Q, ScaleSub#MakeScaleSubPanel()
end

// optional strings are for pre-selecting data or reference waves
static function MakeScaleSubPanel([string ref, string data])
	DoWindow /K ScaleSubPanel
	NewDataFolder /O root:Packages
	NewDataFolder /O root:Packages:ScaleSub
	Make /O root:Packages:ScaleSub:W_preview = NaN
	string /G root:Packages:ScaleSub:NewName = ""
	
	NewPanel /K=1/N=ScaleSubPanel/W=(100,100,1000,500) as "Interactive subtraction"
	ModifyPanel /W=ScaleSubPanel, noEdit=1
	variable fsize = 14
	
	DefineGuide/W=ScaleSubPanel guideDivR={FL,0.45,FR}
	DefineGuide/W=ScaleSubPanel guidePreviewR={FR,-10}
	DefineGuide/W=ScaleSubPanel guidePreviewL={guideDivR, 5}
	DefineGuide/W=ScaleSubPanel guidePreviewT={FT,30}
	DefineGuide/W=ScaleSubPanel guideB={FB,-50}
	DefineGuide/W=ScaleSubPanel guideDataT={FT,0.7,FB}
	DefineGuide/W=ScaleSubPanel guideDataL={FL,10}
	DefineGuide/W=ScaleSubPanel guideRefR={guideDivR,-5}
	DefineGuide/W=ScaleSubPanel guideDivL={guideDataL,0.5,guideRefR}
	DefineGuide/W=ScaleSubPanel guideDataR={guideDivL,-5}
	DefineGuide/W=ScaleSubPanel guideRefL={guideDivL,5}
	
	TitleBox TBoxData win=ScaleSubPanel, pos={15,8}, frame=0, fsize=14, title="Choose data wave:"
	TitleBox TBoxRef win=ScaleSubPanel, pos={215,8}, frame=0, fsize=14, title="Choose reference wave:"
	string strTitle="Preview:           Output = Data - [scale factor] x Reference + [offset]"
	TitleBox TBoxPreview win=ScaleSubPanel, pos={425,8}, frame=0, fsize=14, title=strTitle
		
	ListBox listboxDataWave,pos={10, 30},size={190, 240}, focusRing=0
	ListBox listboxRefWave,pos={210,30},size={190,240}, focusRing=0
	
	MakeListIntoWaveSelector("ScaleSubPanel", "listboxDataWave", content=WMWS_Waves, selectionMode=WMWS_SelectionSingle, listoptions="DIMS:1,TEXT:0")
	MakeListIntoWaveSelector("ScaleSubPanel", "listboxRefWave", content=WMWS_Waves, selectionMode=WMWS_SelectionSingle, listoptions="DIMS:1,TEXT:0")
	WS_SetNotificationProc("ScaleSubPanel", "listboxDataWave", "ScaleSub#UpdateScaleSub", isExtendedProc=1)
	WS_SetNotificationProc("ScaleSubPanel", "listboxRefWave", "ScaleSub#UpdateScaleSub", isExtendedProc=1)
	
	Display/FG=(guidePreviewL, guidePreviewT, guidePreviewR, guideB )/HOST=ScaleSubPanel /N=PreviewPlot root:Packages:ScaleSub:W_preview
	ModifyGraph /W=ScaleSubPanel#PreviewPlot frameStyle=5
	SetWindow ScaleSubPanel#PreviewPlot activeChildFrame=0
	
	Display/FG=(guideDataL, guideDataT, guideDataR, guideB)/HOST=ScaleSubPanel  /N=DataPlot root:Packages:ScaleSub:W_preview
	// temporarily put blank preview wave in here
	ModifyGraph /W=ScaleSubPanel#DataPlot margin(left)=1,margin(bottom)=1,margin(top)=1,margin(right)=1,frameStyle=5
	ModifyGraph /W=ScaleSubPanel#DataPlot noLabel=2, axThick=0
	SetAxis /W=ScaleSubPanel#DataPlot /A bottom
	SetAxis /W=ScaleSubPanel#DataPlot /A left
	SetWindow ScaleSubPanel#DataPlot activeChildFrame=0
	
	Display/FG=(guideRefL, guideDataT, guideRefR, guideB)/HOST=ScaleSubPanel /N=RefPlot root:Packages:ScaleSub:W_preview
	ModifyGraph   /W=ScaleSubPanel#RefPlot margin(left)=1,margin(bottom)=1,margin(top)=1,margin(right)=1,frameStyle=5
	ModifyGraph  /W=ScaleSubPanel#RefPlot noLabel=2, axThick=0
	SetAxis /W=ScaleSubPanel#RefPlot /A bottom
	SetAxis /W=ScaleSubPanel#RefPlot /A left
	SetWindow ScaleSubPanel#RefPlot activeChildFrame=0
	SetActiveSubwindow ScaleSubPanel
	
	SetVariable setvarInc, pos={10,365}, size={140,16}, title="Increment:"
	SetVariable setvarInc, value=_NUM:1, fsize=14, Proc=ScaleSub#ScaleSub_SetVar, limits={-2,Inf,1}
	SetVariable setvarInc, userdata="1", focusRing=0
	
	SetVariable setvarFactor, pos={170,365}, size={160,16}, title="Scale factor:"
	SetVariable setvarFactor, value=_NUM:1, fsize=14, Proc=ScaleSub#ScaleSub_SetVar, focusRing=0
	
	SetVariable setvarOffset, pos={350,365}, size={120,16}, title="Offset:"
	SetVariable setvarOffset, value=_NUM:0, fsize=14, Proc=ScaleSub#ScaleSub_SetVar, focusRing=0

	SetVariable setvarNewName, pos={490,365}, size={340,16}, title="Create this output wave:"
	SetVariable setvarNewName, value=_STR:"new", fsize=14, Proc=ScaleSub#ScaleSub_SetVar, focusRing=0
	Button buttonDoIt, pos={840,365}, size={50,20}, title="Do It", disable=2, Proc=ScaleSub#ScaleSub_Button
	
	// resizing userdata for controls
	TitleBox TBoxData,userdata(ResizeControlsInfo)= A"!!,B)!!#:b!!#@Z!!#<@z!!#N3Bk1ctB6@p!AOKsK@8$Y$zzzzzzzzzzz!!#](Aon\"Qzz"
	TitleBox TBoxData,userdata(ResizeControlsInfo) += A"zzzzzzzzzzzz!!#u:Du]k<zzzzzzzzzzz"
	TitleBox TBoxData,userdata(ResizeControlsInfo) += A"zzz!!#u:DuaGlB6@p!AOKsK@8m4,zzzzzzzzzzz!!!"
	TitleBox TBoxRef,userdata(ResizeControlsInfo)= A"!!,Gg!!#:b!!#A,!!#<@z!!#N3Bk1ctB6@p!AQ!)O9E5%mzzzzzzzzzzz!!#](Aon\"Qzz"
	TitleBox TBoxRef,userdata(ResizeControlsInfo) += A"zzzzzzzzzzzz!!#u:Du]k<zzzzzzzzzzz"
	TitleBox TBoxRef,userdata(ResizeControlsInfo) += A"zzz!!#u:DuaGlB6@p!AOKsK@8m4,zzzzzzzzzzz!!!"
	TitleBox TBoxPreview,userdata(ResizeControlsInfo)= A"!!,I:J,hk8!!#C>!!#<@z!!#N3Bk1ctB6@p!APdDYG%kN39E5%mzzzzzzzzzz!!#r+D.OhkBk2=!z"
	TitleBox TBoxPreview,userdata(ResizeControlsInfo) += A"zzzzzzzzzzzz!!#u:Du]k<zzzzzzzzzzz"
	TitleBox TBoxPreview,userdata(ResizeControlsInfo) += A"zzz!!#u:Du]k<zzzzzzzzzzzzzz!!!"
	ListBox listboxDataWave,userdata(ResizeControlsInfo)= A"!!,A.!!#=S!!#AM!!#B*z!!#](Aon\"Qzzzzzzzzzzzzzz!!#N3Bk1ctB6@p!AOKsK"
	ListBox listboxDataWave,userdata(ResizeControlsInfo) += A"@8[(*zzzzzzzzzzz!!#u:Du]k<zzzzzzzzzzz"
	ListBox listboxDataWave,userdata(ResizeControlsInfo) += A"zzz!!#N3Bk1ctB6@p!AOKsK@8m4,zzzzzzzzzzz!!!"
	ListBox listboxRefWave,userdata(ResizeControlsInfo)= A"!!,Gb!!#=S!!#AM!!#B*z!!#N3Bk1ctB6@p!AQ!)O9E5%mzzzzzzzzzzz!!#N3Bk1ctB6@p!AQ!)O"
	ListBox listboxRefWave,userdata(ResizeControlsInfo) += A";?-[szzzzzzzzzzz!!#u:Du]k<zzzzzzzzzzz"
	ListBox listboxRefWave,userdata(ResizeControlsInfo) += A"zzz!!#N3Bk1ctB6@p!AOKsK@8m4,zzzzzzzzzzz!!!"
	SetVariable setvarInc,userdata(ResizeControlsInfo)= A"!!,A.!!#BpJ,hqF!!#<Xz!!#](Aon\"Qzzzzzzzzzzzzzz!!#](Aon\"Qzz"
	SetVariable setvarInc,userdata(ResizeControlsInfo) += A"zzzzzzzzzzzz!!#?(FEDG<zzzzzzzzzzz"
	SetVariable setvarInc,userdata(ResizeControlsInfo) += A"zzz!!#?(FEDG<zzzzzzzzzzzzzz!!!"
	SetVariable setvarFactor,userdata(ResizeControlsInfo)= A"!!,G:!!#BpJ,hqZ!!#<Xz!!#](Aon\"Qzzzzzzzzzzzzzz!!#](Aon\"Qzz"
	SetVariable setvarFactor,userdata(ResizeControlsInfo) += A"zzzzzzzzzzzz!!#?(FEDG<zzzzzzzzzzz"
	SetVariable setvarFactor,userdata(ResizeControlsInfo) += A"zzz!!#?(FEDG<zzzzzzzzzzzzzz!!!"
	SetVariable setvarOffset,userdata(ResizeControlsInfo)= A"!!,Hj!!#BpJ,hq*!!#<Xz!!#](Aon\"Qzzzzzzzzzzzzzz!!#](Aon\"Qzz"
	SetVariable setvarOffset,userdata(ResizeControlsInfo) += A"zzzzzzzzzzzz!!#?(FEDG<zzzzzzzzzzz"
	SetVariable setvarOffset,userdata(ResizeControlsInfo) += A"zzz!!#?(FEDG<zzzzzzzzzzzzzz!!!"
	SetVariable setvarNewName,userdata(ResizeControlsInfo)= A"!!,I[!!#BpJ,hs:!!#<Xz!!#](Aon\"Qzzzzzzzzzzzzzz!!#o2B4uAezz"
	SetVariable setvarNewName,userdata(ResizeControlsInfo) += A"zzzzzzzzzzzz!!#?(FEDG<zzzzzzzzzzz"
	SetVariable setvarNewName,userdata(ResizeControlsInfo) += A"zzz!!#?(FEDG<zzzzzzzzzzzzzz!!!"
	Button buttonDoIt,userdata(ResizeControlsInfo)= A"!!,Jc!!#BpJ,ho,!!#<Xz!!#o2B4uAezzzzzzzzzzzzzz!!#o2B4uAezz"
	Button buttonDoIt,userdata(ResizeControlsInfo) += A"zzzzzzzzzzzz!!#?(FEDG<zzzzzzzzzzz"
	Button buttonDoIt,userdata(ResizeControlsInfo) += A"zzz!!#?(FEDG<zzzzzzzzzzzzzz!!!"
	
	// resizing userdata for panel
	SetWindow kwTopWin,userdata(ResizeControlsInfo)= A"!!*'\"z!!#Dq!!#C-zzzzzzzzzzzzzzzzzzzzz"
	SetWindow kwTopWin,userdata(ResizeControlsInfo) += A"zzzzzzzzzzzzzzzzzzzzzzzzz"
	SetWindow kwTopWin,userdata(ResizeControlsInfo) += A"zzzzzzzzzzzzzzzzzzz!!!"
	SetWindow kwTopWin,userdata(ResizeControlsGuides)=  "guideDivR;guidePreviewR;guidePreviewL;guidePreviewT;guideB;guideDataT;guideDataL;guideRefR;guideDivL;guideDataR;guideRefL;"
	SetWindow kwTopWin,userdata(ResizeControlsInfoguideDivR)=  "NAME:guideDivR;WIN:ScaleSubPanel;TYPE:User;HORIZONTAL:0;POSITION:405.00;GUIDE1:FL;GUIDE2:FR;RELPOSITION:0.45;"
	SetWindow kwTopWin,userdata(ResizeControlsInfoguidePreviewR)=  "NAME:guidePreviewR;WIN:ScaleSubPanel;TYPE:User;HORIZONTAL:0;POSITION:890.00;GUIDE1:FR;GUIDE2:;RELPOSITION:-10;"
	SetWindow kwTopWin,userdata(ResizeControlsInfoguidePreviewL)=  "NAME:guidePreviewL;WIN:ScaleSubPanel;TYPE:User;HORIZONTAL:0;POSITION:410.00;GUIDE1:guideDivR;GUIDE2:;RELPOSITION:5;"
	SetWindow kwTopWin,userdata(ResizeControlsInfoguidePreviewT)=  "NAME:guidePreviewT;WIN:ScaleSubPanel;TYPE:User;HORIZONTAL:1;POSITION:30.00;GUIDE1:FT;GUIDE2:;RELPOSITION:30;"
	SetWindow kwTopWin,userdata(ResizeControlsInfoguideB)=  "NAME:guideB;WIN:ScaleSubPanel;TYPE:User;HORIZONTAL:1;POSITION:350.00;GUIDE1:FB;GUIDE2:;RELPOSITION:-50;"
	SetWindow kwTopWin,userdata(ResizeControlsInfoguideDataT)=  "NAME:guideDataT;WIN:ScaleSubPanel;TYPE:User;HORIZONTAL:1;POSITION:280.00;GUIDE1:FT;GUIDE2:FB;RELPOSITION:0.7;"
	SetWindow kwTopWin,userdata(ResizeControlsInfoguideDataL)=  "NAME:guideDataL;WIN:ScaleSubPanel;TYPE:User;HORIZONTAL:0;POSITION:10.00;GUIDE1:FL;GUIDE2:;RELPOSITION:10;"
	SetWindow kwTopWin,userdata(ResizeControlsInfoguideRefR)=  "NAME:guideRefR;WIN:ScaleSubPanel;TYPE:User;HORIZONTAL:0;POSITION:400.00;GUIDE1:guideDivR;GUIDE2:;RELPOSITION:-5;"
	SetWindow kwTopWin,userdata(ResizeControlsInfoguideDivL)=  "NAME:guideDivL;WIN:ScaleSubPanel;TYPE:User;HORIZONTAL:0;POSITION:205.00;GUIDE1:guideDataL;GUIDE2:guideRefR;RELPOSITION:0.5;"
	SetWindow kwTopWin,userdata(ResizeControlsInfoguideDataR)=  "NAME:guideDataR;WIN:ScaleSubPanel;TYPE:User;HORIZONTAL:0;POSITION:200.00;GUIDE1:guideDivL;GUIDE2:;RELPOSITION:-5;"
	SetWindow kwTopWin,userdata(ResizeControlsInfoguideRefL)=  "NAME:guideRefL;WIN:ScaleSubPanel;TYPE:User;HORIZONTAL:0;POSITION:210.00;GUIDE1:guideDivL;GUIDE2:;RELPOSITION:5;"

// resizing panel hook
	SetWindow ScaleSubPanel hook(ResizeControls)=ResizeControls#ResizeControlsHook
		
	// we can be flexible with the optional input strings, but WS_SelectAnObject needs full path
	if (ParamIsDefault(ref)==0 && exists(ref)==1)
		wave wref = $ref
		ref = GetWavesDataFolder(wref,2)
		WS_SelectAnObject("ScaleSubPanel", "listboxRefWave", ref, OpenFoldersAsNeeded=1)
		UpdateScaleSub(ref, 0, "ScaleSubPanel", "listboxRefWave")
	endif
	if(ParamIsDefault(data)==0 && exists(data)==1)
		wave wdata = $data
		data = GetWavesDataFolder(wdata,2)
		WS_SelectAnObject("ScaleSubPanel", "listboxDataWave", data, OpenFoldersAsNeeded=1)
		UpdateScaleSub(data, 0, "ScaleSubPanel", "listboxDataWave")
	endif
	
//	WS_UpdateWaveSelectorWidget("ScaleSubPanel", "listboxRefWave")
//	Struct WMWinHookStruct H_Struct
//	WMWS_WinHook(H_Struct)
//	Struct WMListboxAction LB_Struct
//	WaveSelectorListProc(LB_Struct)
	
	
//	ModifyControl listboxRefWave activate

//	controlinfo listboxRefWave
//	UpdateScaleSub(SelectedItem, 0, "ScaleSubPanel", "listboxRefWave")

end

static function UpdateScaleSub(SelectedItem, EventCode, OwningWindowName, ListboxControlName)
	string SelectedItem			// string with full path to the item clicked on in the wave selector
	variable EventCode			// the ListBox event code that triggered this notification
	string OwningWindowName	// String containing the name of the window containing the listbox
	string ListboxControlName	// String containing the name of the listbox control
	
	string refStr, dataStr
	
	if(stringmatch(ListboxControlName, "listboxDataWave"))
		dataStr = SelectedItem
		wave /Z w_data = $RemoveEnding(dataStr, ";")
		RemoveFromGraph /Z/W=ScaleSubPanel#DataPlot $"#0"
		if (WaveExists(w_data))
			// replot wave in small subwindow
			AppendToGraph /W=ScaleSubPanel#DataPlot w_data
			ModifyGraph /W=ScaleSubPanel#DataPlot axThick=0, noLabel=2
			SetAxis  /W=ScaleSubPanel#DataPlot/A bottom
			// reset name suggestion for output
			string tempStr=NameOfWave(w_data)
			
			if(kDefaultOverwrite==0)
				tempStr+="_n"
				if(CheckName(tempStr, 1))
					tempStr=UniqueName(tempStr, 1, 0)
				endif
			endif
		else
			tempStr="new"
		endif
		SetVariable setvarNewName, win=ScaleSubPanel, value=_STR:tempStr
		
		if (exists(tempStr))
			SetVariable setvarNewName, win=ScaleSubPanel, valueBackColor=(65535,0,0)
		else
			SetVariable setvarNewName, win=ScaleSubPanel, valueBackColor=0
		endif
			
		
	endif
	
	if (stringmatch(ListboxControlName, "listboxRefWave"))
		wave /Z w_ref = $RemoveEnding(SelectedItem, ";")
		RemoveFromGraph /Z /W=ScaleSubPanel#RefPlot $"#0"
		if (WaveExists(w_ref))
			AppendToGraph /W=ScaleSubPanel#RefPlot  w_ref
			ModifyGraph /W=ScaleSubPanel#RefPlot axThick=0, noLabel=2
			SetAxis  /W=ScaleSubPanel#RefPlot  /A bottom
		endif
	endif
	
	doSub()
end

static function doSub()
	
	string refStr, dataStr
	refStr = WS_SelectedObjectsList("ScaleSubPanel", "listboxRefWave")
	dataStr = WS_SelectedObjectsList("ScaleSubPanel", "listboxDataWave")
	
	refStr = RemoveEnding(refStr, ";")
	dataStr = RemoveEnding(dataStr, ";")
	
	wave w_preview = root:Packages:ScaleSub:w_preview
	
	wave /Z w_ref = $refStr
	wave /Z w_data = $dataStr
		
	string msg=""
	if (!WaveExists(w_ref))
		msg+="no reference wave selected\r"
	endif
	if (!WaveExists(w_data))
		msg+="no data wave selected\r"
	endif
	
	TextBox /C/F=0/W=ScaleSubPanel#PreviewPlot /N=text0/Z=1/X=5.00/Y=5.00/A=LT  "\\K(26112,52224,0)"+msg
	
	if (strlen(msg)==0)
		Duplicate /o w_data w_preview
		ControlInfo /W=ScaleSubPanel setvarFactor
		variable scale=v_value
		ControlInfo /W=ScaleSubPanel setvarOffset
		variable offset=v_value
		ControlInfo /W=ScaleSubPanel setvarNewName
		string newStr=s_value
		
		w_preview = w_data - offset - scale*refValue(w_ref, x)
			
		Button buttonDoIt win=ScaleSubPanel, disable=0
		if (offset)
			sprintf msg, "%s = %s % + g %+g * %s", newStr, NameOfWave(w_data), offset, -scale, NameOfWave(w_ref)
		else
			sprintf msg, "%s = %s %+g * %s", newStr, NameOfWave(w_data), -scale, NameOfWave(w_ref)
		endif
		TextBox /C/F=0/W=ScaleSubPanel#PreviewPlot /N=text0/Z=1/X=5.00/Y=5.00/A=LT  "\\K(26112,52224,0)"+msg
	
		return 1
	endif
	w_preview = NaN
	//SetVariable  setvarNewName, win=ScaleSubPanel, value=_STR:"new"
	Button buttonDoIt win=ScaleSubPanel , disable=2
end

static function refValue(wave w_ref, variable v_x)
	Make /free/N=2 w_range
	w_range[0] = leftx(w_ref)
	w_range[1] = pnt2x(w_ref,numpnts(w_ref)-1)
	if (v_x<WaveMin(w_range) || v_x > WaveMax(w_range))
		return NaN
	endif
	return w_ref(v_x)
end

static function ScaleSub_Button(STRUCT WMButtonAction &s)
	if (s.eventCode == -1)
		// won't work if preview wave is plotted
		KillWindow /Z ScaleSubPanel#PreviewPlot
		KillWindow /Z ScaleSubPanel#DataPlot
		KillWindow /Z ScaleSubPanel#RefPlot
		KillDataFolder /Z root:Packages:ScaleSub
		return 0
	endif
	
	if (s.eventCode != 2)
		return 0
	endif
	
	wave w_preview=root:Packages:ScaleSub:W_preview
	
	ControlInfo /W=ScaleSubPanel setvarNewName
	string newStr=s_value
	
	if (CheckName(newStr, 1) != 0)
		DoAlert 1, ParseFilePath(0, newStr, ":", 1, 0)+" exists. Overwrite?"
		if (V_flag == 2)
			return 0
		endif
	endif
	Duplicate /o w_preview $newStr
	
	string s_txt = StringByKey("TEXT",AnnotationInfo("ScaleSubPanel#PreviewPlot", "text0"))
	printf "•%s\r", s_txt[18,Inf]
	
	// update wave selector listboxes so that new wave appears
	WS_UpdateWaveSelectorWidget("ScaleSubPanel", "listboxRefWave")
	WS_UpdateWaveSelectorWidget("ScaleSubPanel", "listboxDataWave")
	return 1
end

static function ScaleSub_SetVar(STRUCT WMSetVariableAction &s)
	if(s.eventCode<0) // excludes Igor8 eventcodes -2 and -3
		return 0
	endif
	
	if(stringmatch(s.ctrlName, "setvarInc"))
		
		ControlInfo /W=ScaleSubPanel setvarInc
		variable old=str2num(S_UserData)
		variable new=s.dval
		
		if (abs(new-old) < 1e-15)
			return 0
		endif
		
		if (numtype(new) != 0)
			return 0
		endif
		
		if (s.eventCode == 1) // mouseup = increment button
			if (new < old) // trying to decrement
				new = 10^(log(old-1e-15) - 1)
			else // trying to increment
				new = 10^(log(old) + 1)
			endif
			SetVariable setvarInc, win=ScaleSubPanel, value=_NUM:new
		endif
		
		SetVariable setvarFactor, win=ScaleSubPanel, limits={-Inf,Inf,new}
		SetVariable setvarOffset, win=ScaleSubPanel, limits={-Inf,Inf,new}
		
		//store current value for future use
		SetVariable setvarInc, win=ScaleSubPanel, userdata=num2str(new)
	endif
	
	if (stringmatch(s.ctrlName, "setvarNewName"))
		if (exists(s.sval))
			SetVariable $s.ctrlName, win=$s.win, valueBackColor=(65535,0,0)
		else
			SetVariable $s.ctrlName, win=$s.win, valueBackColor=0
		endif
	endif
	
	doSub()
	
	return 0
end